function [y1,y2]=biwav1(x1,x2,x3,x4,x5)
%BIWAV1 Biorthogonal regular linear phase filter design
% [H0,H1]=BIWAV1(L,K,B,p) finds biorthogonal low-pass filter pair (H0,H1)
% such that:
% - H0(z) is a linear-phase equiripple filter of length L having K zeroes
%  at z==-1, normalized transition bandwidth B and pass-band weight p
%  (as determined by ZREMEZ)
% - H1(z) is a linear-phase filter of length L+2*(K-1) having K zeroes at
%  z==-1 that minimizes the quadratic distance between H0 and H1
%  (closest to the orthogonal case).
%
% BIWAV1(L,K,B) first estimates p on the orthogonal equivalent of H0
% using REMEZWAV.
%
% See also BIWAV2, BIWAV3, CO, REMEZWAV, ZREMEZ in the SIMPA Toolbox.

% For MATLAB 4.1
% Author: Olivier Rioul May 11 1994

% References:
% H. Le Bihan, P. Siohan, O. Rioul, P. Duhamel, "Une mthode simple de calcul
% de bancs de filtres/ondelettes bi-orthogonales", GRETSI'93, pp. 443-446.

global w;

if nargin==5 % call biwav1(dw,L,K,B,p)
 dw=x1;L=x2;K=x3;B=x4;p=x5;
 [h,w]=zremez(L,K,B,dw,p,w); h=h*sqrt(2);
 hp=co(h,K);
 L=length(h); Lp=length(hp);
 if L<Lp
  m=(Lp-L)/2; z=zeros(1,m); err=norm(hp-[z h z]);
 else
  m=(L-Lp)/2; z=zeros(1,m); err=norm(h-[z hp z]);
 end
 y1=err;
%err,dw
%magres(h/sqrt(2),mirror(hp)/sqrt(2),'dB'); drawnow


else % call biwav1(L,K,B,p)
 L=x1;K=x2;B=x3;
 if nargin==4, p=x4;
 else % nargin==3
  [dum1,dum2,delta]=remezwav(L,K,B);
  delta1=1-sqrt(1-delta); 
  delta2=sqrt(delta);     %  ou  2*sqrt(delta) ?
  p=delta2/delta1;
  fprintf('Pass-band weight = %g\n',p);
 end
 w=0; % reset w
 options=foptions; options(14)=100;
 dw=fmin('biwav1',B/2-0.05,0.1-B/2,options,L,K,B,p);
 [h,w]=zremez(L,K,B,dw,p,w); h=h*sqrt(2); hp=co(h,K);
 y1=h; y2=hp; 
end
