function [y1,y2]=biwav2(x1,x2,x3,x4)
%BIWAV2 Biorthogonal regular linear phase filter design
% [H0,H1]=BIWAV2(L,K,B) finds biorthogonal low-pass filter pair (H0,H1)
% such that:
% - H0(z) is a linear-phase equiripple filter of length L having K zeroes
%  at z==-1 and normalized transition bandwidth B (as determined by ZREMEZ)
% - H1(z) is a linear-phase filter of length L+2*(K-1) having K zeroes at
%  z==-1 that minimizes the quadratic distance between H0 and H1
%  (closest to the orthogonal case).
%
% See also BIWAV1, BIWAV3, CO, REMEZWAV, ZREMEZ in the SIMPA Toolbox.

% For MATLAB 4.1
% Author: Olivier Rioul May 11 1994

% References:
% H. Le Bihan, P. Siohan, O. Rioul, P. Duhamel, "Une mthode simple de calcul
% de bancs de filtres/ondelettes bi-orthogonales", GRETSI'93, pp. 443-446.

global w;

if nargin==4 % call biwav2([dw p],L,K,B)
 dw=x1(1); p=x1(2); L=x2;K=x3;B=x4;

 % protections:
 p=abs(p);
 if p<1e-4, p=1e-4; end
 if .25+B/2+dw>=.5, dw=.495-.25-B/2; end
 if .25-B/2+dw<=0, dw=0.05-.25+B/2; end
 
 [h,w]=zremez(L,K,B,dw,p,w); h=h*sqrt(2);
 hp=co(h,K);
 L=length(h); Lp=length(hp);
 if L<Lp
  m=(Lp-L)/2; z=zeros(1,m); err=norm(hp-[z h z]);
 else
  m=(L-Lp)/2; z=zeros(1,m); err=norm(h-[z hp z]);
 end
 y1=err;
%err,dw,p
%magres(h/sqrt(2),mirror(hp)/sqrt(2),'dB'); drawnow


else % call biwav2(L,K,B)
 L=x1;K=x2;B=x3;
 w=0; % reset w
 options=foptions; options(14)=200;
 dw0=0; p0=1;
 x=fmins('biwav2',[dw0,p0],options,[],L,K,B);
 dw=x(1); p=x(2);
 [h,w]=zremez(L,K,B,dw,p,w); h=h*sqrt(2); hp=co(h,K);
 y1=h; y2=hp; 
end
