function [y1,y2]=biwav3(x1,x2,x3,x4)
%BIWAV3 Biorthogonal regular linear phase filter design
% [H0,H1]=BIWAV3(L,K,ws) finds biorthogonal low-pass filter pair (H0,H1)
% such that:
% - H0(z) is a linear-phase equiripple filter of length L having K zeroes
%  at z==-1 and normalized stop-band cut-off frequency ws (determined by
%  ZREMEZ)
% - H1(z) is a linear-phase filter of length L+2*(K-1) having K zeroes at
%  z==-1 such that both H0 and H1 are maximally frequency selective.
%
% See also BIWAV1, BIWAV2, CO, REMEZWAV, ZREMEZ in the SIMPA Toolbox.

% For MATLAB 4.1
% Author: Olivier Rioul May 11 1994

% References:
% H. Le Bihan, P. Siohan, O. Rioul, P. Duhamel, "Une mthode simple de calcul
% de bancs de filtres/ondelettes bi-orthogonales", GRETSI'93, pp. 443-446.

global w wstop
N=512; %FFT

if nargin==4 % call biwav3([dw p B],L,K,1)
 dw=x1(1); p=x1(2); B=x1(3); L=x2;K=x3;

 % protections:
 B=abs(B);
 if B<1e-3, B=1e-3; end
 p=abs(p);
 if p<1e-4, p=1e-4; end
 if .25+B/2+dw>=.5, dw=.495-.25-B/2; end
 if .25-B/2+dw<=0, dw=0.05-.25+B/2; end
 
 [h,w]=zremez(L,K,B,dw,p,w); h=h*sqrt(2);
 hp=co(h,K);

 H=fft(h,N);  H=abs(H(1:N/2));
 HP=fft(hp,N); HP=abs(HP(1:N/2));
 delta=max(H(wstop:N/2)); deltap=max(HP(wstop:N/2));
 err=delta+deltap;
 y1=err;
%delta,deltap,err,dw,p,B
%magres(h/sqrt(2),mirror(hp)/sqrt(2),'dB',-60); drawnow


else % call biwav3(L,K,ws)
 L=x1;K=x2;ws=x3;
 w=0; wstop=round(ws*N); % reset w and wstop
 options=foptions; options(14)=300;
 dw0=0; p0=1; 
 B0=abs(ws-0.25);
 if B0<0.03, B0=0.03; end
 x=fmins('biwav3',[dw0,p0,B0],options,[],L,K,1);
 dw=x(1); p=x(2); B=x(3);
 [h,w]=zremez(L,K,B,dw,p,w); h=h*sqrt(2); hp=co(h,K);
 y1=h; y2=hp; 
end
