function h=factor(z,type)
%FACTOR Zero factoring (Riesz lemma)
% FACTOR(Z) produces filter H by factoring the complex zeroes Z
% of the linear phase filter SQUARE(H) (as obtained by design routines).
% Zeroes in Z must have even multiplicity on the unit circle and
% be associated in (z,z*,1/z,1/z*) quadruples outside the unit circle.
% The resulting filter H is normalized such that NORM(H)=1.
%
% By default, selection of zeros (z,z*) or (1/z, 1/z*) for H is done
% manually by clicking the mouse on the screen.
% FACTOR(Z,'min') gives the minimum phase solution H
% FACTOR(Z,'max') gives the maximum phase solution H
% FACTOR(Z,'lin') gives the solution whose phase response is closest to
% linear (as determined by PHASEVAR)
%
% FACTOR(Z,'all') gives all solutions as lines of matrix H=[H0; H1; H2; ... ]
% With the convention that indexes range from 0, Hn is designed by selecting
% zeros outside (1) or inside (0) the unit disk as absolute angle of zeros
% increases. For example, n=1011 in base 2 means (in reverse order):
% First outside, then outside, then inside, then outside. 
% H0 is the minimum-phase solution, while last Hn (n=2^(..)-1) is the
% maximum-phase solution. Note that the second half of solutions are the
% time-reversals of the first half.
%
% FACTOR(Z,'allsorted') gives all solutions sorted from increasing
% PHASEVARs as lines of matrix H
%
% CAUTION: It works properly up to length 20.
%
% See also PHASEVAR, REMEZWAV, MAGRES, SQUARE, ZPLOT in the SIMPA Toolbox
% and NORM in the standard MATLAB Toolbox

% For MATLAB 4.1
% Author: Olivier Rioul March 15 1994

% References:
% Mark J. T. Smith and Thomas P. Barnwell III, Exact reconstruction
%    techniques for tree-structured subband coders, IEEE Trans. ASSP,
%    Vol. 34, No. 3, pp. 434-441, 1986.
% Ingrid Daubechies, Orthonormal bases of compactly supported wavelets,
%    Comm. Pure Appl. Math., Vol. XLI, No. 7, pp. 909-996, 1988
% Olivier Rioul and Pierre Duhamel, A Remez exchange algorithm for
%    orthonormal wavelets, IEEE Trans. CAS II, To appear.

if min(size(z))~=1, error('input must be a vector'), end
z=z(:);
tolz=0.05; % tolerance for zeros
L=length(z)/2+1; 
% length of filter h: this is needed when one computes all solutions
% (see below)

% zeros at z=-1: zm1
I=find(abs(z+1)<2.5*tolz); z(I)=[];
if rem(length(I),2), error('root -1 has odd multiplicity'), end
zm1=-ones(length(I)/2,1);

% zeros on unit circle ~=-1: zcirc
zcirc=z(abs(abs(z)-1)<tolz);
if ~isempty(zcirc)
 % test multiplicity on unit circle
 [dummy,I]=sort(angle(zcirc)); zcirc=zcirc(I); % sort by ascending angles
 lzc=length(zcirc); zc1=zcirc(1:2:lzc); zc2=zcirc(2:2:lzc);
 if max(abs(zc1-zc2))>tolz, error('roots on unit circle have odd multiplicity'), end
 zcirc=(zc1+zc2)/2;
end

% zeros outside unit circle: z
z=z(abs(abs(z)-1)>=tolz); 
% test if linear phase
p=poly(z);
if max(imag(p))>0.05, error('roots are not of a real filter'); end
p=real(p);
if max(p-fliplr(p))>0.25, error('roots are not a linear phase filter'); end

% Nb of choices = # outcircle (complex roots/4 + real roots/2)
nreal=sum(abs(imag(z))<tolz); % # real zeros
nch=nreal/2+(length(z)-nreal)/4;
if nch~=round(nch), error('Bug in factor'); end

if nargin==1, type=0;
elseif strcmp(type,'min'), type=1;
elseif strcmp(type,'max'), type=2;
elseif strcmp(type,'all'), type=3;
elseif strcmp(type,'allsorted'), type=4;
elseif strcmp(type,'lin'), type=5;
else error('type is not valid.')
end

if type<=2
%%%%%%%%%%%%%%%%% MOUSE SELECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if type==0
 % plot
 x=linspace(0,2*pi); zz=[z;zcirc];
 plot(cos(x),sin(x),'-m',real(zz),imag(zz),'oy');
 title('CLICK TO SELECT ZEROS'); axis('equal'); hold on
 if ~isempty(zm1), plot(-1,0,'oc'); end

 % selection of zero zz in z, put in zh
 zh=[];
 for n=1:nch
  [x,y]=ginput(1); zz=x+j*y; % get zero zz
  [dummy,I]=min(abs(z-zz)); I=I(1); 
  zz=z(I);  zh=[zh; zz]; % closest to zero in z
  % remove roots in z
  z(I)=[]; 
  [dummy,I]=min(abs(z-1/zz)); I=I(1); z(I)=[];
  if abs(imag(zz))>tolz % complex root
   zh=[zh; conj(zz)];
   [dummy,I]=min(abs(z-conj(zz))); I=I(1); z(I)=[];
   [dummy,I]=min(abs(z-conj(1/zz))); I=I(1); z(I)=[];
  end
  plot([real(zz) real(zz)],[imag(zz) -imag(zz)],'*g');
  plot([real(1/zz) real(1/zz)],[imag(1/zz) -imag(1/zz)],'*r');
 end
 hold off
%%%%%%%%%%%%%%%%%% SPECIAL TYPES %%%%%%%%%%%%%%%%%%%%%%%%%%%% 
elseif type==1, zh=z(abs(z)<1);
elseif type==2, zh=z(abs(z)>1);
end
%%%%%%%%%%%%%%%%%%%%% COMPUTE FILTER %%%%%%%%%%%%%%%%%%%%%%%%
z=[zm1; zcirc; zh];
h=real(poly(z)); 
if sum(h)<0, h=-h; end
h=h/norm(h);
return
end % type<=2

%%%%%%%%%%%%%% COMPUTE ALL SOLUTIONS %%%%%%%%%%%%%%%%%%%%%%%%
nsol=2^nch; % nch: #choices, nsol:#solutions
zreal=real(z(abs(imag(z))<tolz)); % real zeros
z=z(abs(imag(z))>=tolz); % complex zeros
% sort them in ascending abs(angle)
[dummy,I]=sort(abs(angle(z))); z=z(I); 
z=real(z)+i*abs(imag(z)); Lz=length(z);
z=2\(z(1:2:Lz)+z(2:2:Lz)); % upperhalf of complex plane only
z=[zreal(zreal>0); z; zreal(zreal<0)]; % add real zeros
z=[z(abs(z)<1) z(abs(z)>1)]; % left: inside, right: outside

h=zeros(nsol/2,L); % see above L=length of filter h
for nn=0:nsol/2-1
 zh=zeros(nch,1); n=nn;
 for i=1:nch
  zh(i)=z(i,rem(n,2)+1); n=fix(n/2); % select inside/outside
 end
 zreal=zh(abs(imag(zh))<tolz); % take out real zeros
 zh=zh(abs(imag(zh))>=tolz);
 zh=[zreal; zh; conj(zh)]; % gather + conjugates
 zh=[zm1; zcirc; zh];      % gather + on circle
 hh=real(poly(zh));        % compute filter
 if sum(hh)<0, hh=-hh; end
 h(nn+1,:)=hh/norm(hh);    % store filter
end
h=[h;flipud(fliplr(h))];   % add reversed solutions

if type==4 | type==5 % sort by phasevars
 [dummy,I]=sort(phasevar(h)); h=h(I,:);
end
if type==5, h=h(1,:); end

