function [p,r]=quadwav(L,K,B)
%QUADWAV Optimal wavelet obtained using quadratic approximation.
% QUADWAV(L,K,B) gives impulse response of P(z), product filter of para-
% unitary filter bank solution H(z) of length L satisfying K flatness
% constraints (wavelet filter), such that energy is minimized in the stop band.
% The normalized transition bandwidth is given as B. This argument is
% optional if K==L/2 (Daubechies filter).
% P(z) is computed using linear programming (Dantzig algorithm).
%
% [P,R]=QUADWAV(L,K,B) also gives the roots of P(z), which
% can be used to determine H(z).

% For MATLAB 4.1
% Author: Olivier Rioul Oct 16 1992.
% Last modified: Olivier Rioul May 10 1994.

% References:
% Olivier Rioul, Ondelettes regulieres: Application a la compression d'images
%     fixes, PhD Thesis, Telecom Paris, March 1993.

if rem(L,2),error('L must be even');end
if rem(L/2-K,2), K=K+1, end
if K>L/2, error('K is too large'), end
if K==L/2, [p,r]=remezwav(L,L/2); return; end
N=L/2-K; % degrees of freedom

% Initial Daubechies solution of length L
binom=pascal(L,1);
x0=binom(L,1:L/2)./(L-1:-2:1);
x0=fliplr(x0)/sum(x0); x0=x0(:);

wp=2*pi*(1/4-B/2); % pass-band cut-off frequency
gridens=max(60,10*N);       % grid density
gap=2*pi*K/100;    % avoid constraints near w=0
grid=sqrt(linspace(gap,wp^2,gridens)); % non uniform grid

% Equality constraints
A=[]; b=[];
if K>0
 b=zeros(K,1); b(1)=1;
 for i=0:K-1
  a=[1:2:L-1].^(2*i);
  a=a./sum(a)*L/2;
  A=[A;a];
 end
end

% Inequality constraints
A=[A;cos(grid'*(1:2:L-1))];
b=[b;ones(gridens,1)];

% slack variables
y=b-A*x0; y(1:K)=[];

% Dirty trick: add offset=10 such that variables stay positive
offset=10*ones(size(x0));
b=b+A*offset; x0=x0+offset;

% initialisation of Dantzig algorithm
A=[A [zeros(K,gridens);eye(gridens)]]; % constraint matrix
x=[x0; y];                             % initial point
c=sin((1:2:L-1)*wp)./(1:2:L-1);        % cost function c*x
c=[c zeros(1,gridens)];


% SIMPLEX ALGORITHM
nonbase=L/2+1:L-K;               % indexes for base (column vectors of A)
base=[1:L/2 L-K+1:L/2+gridens];  % indexes for non base
Ainv=inv(A(:,base));
iter=0;

while 1
 iter=iter+1;
 if iter>130
  disp('Too many iterations. Result may be inaccurate'); 
  break;
 end

 % one column of alpha = coeff of basis decomposition of one nonbase vector
 % alpha=A(:,base)\A(:,nonbase);
 alpha=Ainv*A(:,nonbase);

 % marginal costs for each non-base vector
 [mcost,j]=max(c(nonbase)-c(base)*alpha);
 % nonbase(j) is the index of ingoing vector (maximizes mcost)

 if mcost<=0, break; end 
 % we have converged (cannot increase cost anymore)

 alpha=alpha(:,j);
 if all(alpha<0), error('infinite solution'); end
 % increase cost indefinitely

 % find theta=min x(i)/alpha(i,j) for alpha(i,j)>0
 % remove zero and negative values.
 ind=find(alpha>0);
 y=x(base(ind))./alpha(ind);
 [theta,i]=min(y); i=ind(i);
 % base(i) is the index of outgoing vector

 % update current point
 x(base)=x(base)-theta*alpha; % note that x(base(i))==0
 x(nonbase(j))=theta;

 % update indexes (switch ingoing/outgoing)
 ibase=base(i);
 base(i)=nonbase(j);
 nonbase(j)=ibase;
 
 % factor for inverse of A(:,base) = Ainv
 E=eye(gridens+K);
 E(:,i)=Ainv*A(:,base(i)); 
 Ainv=E\Ainv;

end

fprintf('%g iterations in quadwav.\n',iter);

% reshape result
a=x(1:L/2)-offset; 
a=a(:)';
p=zeros(1,2*L-1);
p(1:2:2*L-1)=[fliplr(a) a]/2; p(L)=1;
if nargout>1, r=roots(p); end
